angular.module('WebUI').factory('AuthenticationService', ['$window', function ($window) {
    var AuthenticationService = {
        isLogged: function () {
            return angular.isDefined($window.sessionStorage.token);
        },

        isAdminLogged: function () {
            return AuthenticationService.isLogged() && $window.sessionStorage.type === 'admin';
        },

        isUserLogged: function () {
            return AuthenticationService.isLogged() && $window.sessionStorage.type === 'user';
        }
    };

    return AuthenticationService;
}]);

angular.module('WebUI').factory('UserService', ['$q', '$window', '$location', '$http', 'LogoutTimer', 'Status', 'Model', 'Config', function ($q, $window, $location, $http, LogoutTimer, Status, Model, Config) {
    var UserService = {
        login: function (password) {
            var deferred = $q.defer();

            $http.post('Login', {password: password})
                .then(function (response) {
                    if (response && response.data && response.data.token && response.data.type) {
                        $window.sessionStorage.token = response.data.token;
                        $window.sessionStorage.type = response.data.type;

                        Model.removeAll(true);
                        Model.getAll()
                            .then(function () {
                                var defaultUrl;

                                Status.keepAlive.start();

                                switch ($window.sessionStorage.type) {
                                    case 'admin':
                                        defaultUrl = Model.parameters.DefaultURLForAdmin.value;
                                        break;

                                    case 'user':
                                        defaultUrl = Model.parameters.DefaultURLForUser.value;
                                        break;

                                    default:
                                        defaultUrl = Config.defaultUrl;
                                        break;
                                }

                                $location.path(defaultUrl);
                                deferred.resolve(response);
                            })
                            .catch(deferred.reject);
                    } else {
                        deferred.reject(response);
                    }
                })
                .catch(deferred.reject);

            return deferred.promise;
        },

        logout: function () {
            return LogoutTimer.logout();
        }
    };

    return UserService;
}]);

angular.module('WebUI').factory('Tools', ['$locale', 'Config', function ($locale, Config) {
    var Tools = {
        checkFlag: function (flags, mask) {
            if (angular.isDefined(flags) && angular.isDefined(mask)) {
                var parsedFlags = Number(flags);
                var parsedMask = Number(mask);

                if (!isNaN(parsedFlags) && !isNaN(parsedMask)) {
                    return (parsedFlags & parsedMask) === parsedMask;
                }
            }

            return false;
        },

        getPropertyByString: function (object, property) {
            property = property.replace(/\[(\d+)\]/, '.$1');

            var parts = property.split('.'),
                current = parts[0],
                i = 1;

            while ((object = object[current]) && i < parts.length) {
                current = parts[i];
                i++;
            }

            return object;
        },

        swapArrayElements: function (array, value, where) {
            var oldIndex = array.indexOf(value);
            var newIndex = oldIndex + where;

            if (oldIndex !== -1 && newIndex >= 0 && newIndex < array.length) {
                var tmp = array[oldIndex];
                array[oldIndex] = array[newIndex];
                array[newIndex] = tmp;
            }
        },

        mergeObjectsByKeyName: function (dstObj, srcObj, key) {
            var keys = angular.isArray(key) ? key : [key];

            angular.forEach(srcObj, function (value, key) {
                if (!angular.isString(key) || key.search(/^\$.*$/) === -1) {
                    if (keys.indexOf(key) !== -1) {
                        dstObj[key] = angular.copy(value);
                    } else if (angular.isObject(value)) {
                        if (angular.isUndefined(dstObj[key])) {
                            if (angular.isArray(value)) {
                                dstObj[key] = [];
                            } else if (angular.isObject(value)) {
                                dstObj[key] = {};
                            }
                        }

                        Tools.mergeObjectsByKeyName(dstObj[key], value, keys);
                    }
                }
            });
        },

        updateLocales: function (text) {
            angular.forEach($locale.DATETIME_FORMATS.MONTH, function (month, id) {
                $locale.DATETIME_FORMATS.MONTH[id] = text['Month:' + id];
            });

            angular.forEach($locale.DATETIME_FORMATS.SHORTDAY, function (shortDay, id) {
                $locale.DATETIME_FORMATS.SHORTDAY[id] = text['ShortDay:' + id];
            });
        },

        getFileNameFromPath: function (path) {
            var match = path.match(/^(.*\/)?(.+)$/);
            return match ? match[2] : '';
        },

        getFileExtensionFromFileName: function (name) {
            var match = name.match(/^.*\.(.+)$/);
            return match ? match[1] : '';
        },

        List: {
            copyEntry: function (dstEntry, srcEntry) {
                angular.forEach(srcEntry, function (value, key) {
                    if (angular.isObject(dstEntry[key]) && dstEntry[key].hasOwnProperty('value')) {
                        dstEntry[key].value = value;
                    } else {
                        dstEntry[key] = value;
                    }
                });
            },

            isEntryConfigured: function (entry, defaultEntry) {
                var entryConfigured = false;

                angular.forEach(defaultEntry, function (value, key) {
                    if (entry[key].value !== value) {
                        entryConfigured = true;
                    }
                });

                return entryConfigured;
            },

            isEntryLocked: function (entry) {
                var entryLocked = true;

                angular.forEach(entry, function (value) {
                    if (angular.isDefined(value.flags) && !Tools.checkFlag(value.flags, Config.flags.ui_lock)) {
                        entryLocked = false;
                    }
                });

                return entryLocked;
            },

            getConfiguredEntries: function (list, defaultEntry) {
                return (angular.isArray(list) ? list.filter(function (entry) {
                    return Tools.List.isEntryConfigured(entry, defaultEntry);
                }) : []);
            },

            getEditableEntries: function (list, defaultEntry) {
                return (angular.isArray(list) ? Tools.List.getConfiguredEntries(list, defaultEntry).filter(function (entry) {
                    return !Tools.List.isEntryLocked(entry);
                }) : []);
            },

            getFreeEntryIndex: function (list, defaultEntry) {
                var freeEntryIndex = null;

                if (angular.isArray(list)) {
                    list.some(function (entry, index) {
                        var freeEntry = !Tools.List.isEntryConfigured(entry, defaultEntry) && !Tools.List.isEntryLocked(entry);

                        if (freeEntry) {
                            freeEntryIndex = index;
                        }

                        return freeEntry;
                    });
                }

                return freeEntryIndex;
            },

            deleteEntry: function (entry, defaultEntry) {
                if (!Tools.List.isEntryLocked(entry)) {
                    Tools.List.copyEntry(entry, defaultEntry);
                }
            },

            deleteAllEntries: function (list, defaultEntry) {
                angular.forEach(list, function (entry) {
                    Tools.List.deleteEntry(entry, defaultEntry);
                });
            },

            prepareSingleEntry: function (list, index) {
                var singleEntry = [];

                angular.forEach(list, function (entry, entryId) {
                    singleEntry[entryId] = (index !== entryId) ? {} : list[entryId];
                });

                return singleEntry;
            },

            hasUniqueEntries: function (list, defaultEntry, compareObject, compareObjectHasValues) {
                var configuredEntries = Tools.List.getConfiguredEntries(list, defaultEntry);
                var entriesToCompare = configuredEntries.map(function (entry) {
                    var entryToCompare = {};

                    angular.forEach(compareObject, function (value, key) {
                        entryToCompare[key] = entry[key].value;
                    });

                    return entryToCompare;
                });

                if (compareObjectHasValues) {
                    return entriesToCompare.every(function (entry) {
                        return !angular.equals(entry, compareObject);
                    });
                } else {
                    return entriesToCompare.every(function (entryOuter, indexOuter) {
                        return entriesToCompare.every(function (entryInner, indexInner) {
                            return (indexOuter !== indexInner) ? !angular.equals(entryOuter, entryInner) : true;
                        });
                    });
                }
            }
        },

        DelimitedString: {
            getArray: function (string, separator) {
                separator = separator || ',';

                return string
                    .split(separator)
                    .map(function (element) {
                        return element.trim();
                    })
                    .filter(function (element) {
                        return element.length !== 0;
                    });
            },

            add: function (string, element, separator) {
                separator = separator || ',';

                var delimitedStringArray = Tools.DelimitedString.getArray(string, separator);
                delimitedStringArray.push(element);

                return delimitedStringArray.join(separator);
            },

            remove: function (string, element, separator) {
                separator = separator || ',';

                var delimitedStringArray = Tools.DelimitedString.getArray(string, separator);

                if (delimitedStringArray.indexOf(element) !== -1) {
                    delimitedStringArray.splice(delimitedStringArray.indexOf(element), 1);
                    return delimitedStringArray.join(separator);
                } else {
                    return string;
                }
            },

            exist: function (string, element, separator) {
                separator = separator || ',';

                return Tools.DelimitedString.getArray(string, separator).indexOf(element) !== -1;
            }
        }
    };

    return Tools;
}]);

angular.module('WebUI').factory('Popup', ['$q', '$modal', '$log', 'LogoutTimer', 'Config', function ($q, $modal, $log, LogoutTimer, Config) {
    var data = [];
    data.initLayer = function (layer) {
        if (!data[layer]) {
            data[layer] = {
                modalInstance: null,
                priority: 0,
                id: 0
            };
        }
    };

    var Popup = {
        show: function (popupData, config) {
            var deferred;

            config.priority = config.priority || Config.popup.priority.info;
            config.force = config.force || false;
            config.chain = config.chain || false;
            config.closable = config.closable || false;
            config.layer = config.layer || Config.popup.layer.default;

            data.initLayer(config.layer);

            if (!data[config.layer].modalInstance) {
                if (!angular.isString(popupData.message)) {
                    $log.warn('Popup is not displayed because text is missing.');
                    $log.debug('  popupData: %o', popupData);
                    $log.debug('  config: %o', config);
                    $log.debug('  data: %o', data);

                    return $q.reject(Config.popup.error.dataIsMissing);
                } else {
                    data[config.layer].modalInstance = $modal.open({
                        templateUrl: 'templates/Popup.html',
                        controller: 'PopupCtrl',
                        size: config.size || 'md',
                        backdrop: config.closable || 'static',
                        windowClass: 'layer-' + config.layer,
                        openedClass: 'modal-layer-' + config.layer,
                        keyboard: config.closable,
                        resolve: {
                            data: function () {
                                return angular.copy(popupData);
                            }
                        }
                    });

                    data[config.layer].modalInstance.opened
                        .then(function () {
                            data[config.layer].priority = config.priority;
                            popupData.id = ++data[config.layer].id;
                            popupData.layer = config.layer;

                            LogoutTimer.restart.disabled = true;
                            LogoutTimer.cancel();

                            $log.info('Popup is displayed.');
                            $log.debug('  popupData: %o', popupData);
                            $log.debug('  config: %o', config);
                            $log.debug('  data: %o', data);
                        });

                    data[config.layer].modalInstance.result
                        .finally(function () {
                            data[config.layer].modalInstance = null;
                            data[config.layer].priority = 0;

                            if (!Popup.isDisplayed()) {
                                LogoutTimer.restart.disabled = false;
                                LogoutTimer.restart();
                            }
                        });

                    return data[config.layer].modalInstance.result;
                }
            } else if (config.priority > data[config.layer].priority || config.force || config.chain) {
                deferred = $q.defer();

                data[config.layer].modalInstance.result
                    .finally(function () {
                        Popup.show(popupData, config)
                            .then(deferred.resolve, deferred.reject);
                    });

                if (!config.chain) {
                    Popup.dismiss(config.layer);
                }

                $log.info('New popup will be displayed when previous one is closed.');
                $log.debug('  popupData: %o', popupData);
                $log.debug('  config: %o', config);
                $log.debug('  data: %o', data);

                return deferred.promise;
            } else {
                $log.info('Popup is not displayed because another popup is shown.');
                $log.debug('  popupData: %o', popupData);
                $log.debug('  config: %o', config);
                $log.debug('  data: %o', data);

                return $q.reject(Config.popup.error.otherPopupIsShown);
            }
        },

        close: function (layer, id) {
            if (angular.isNumber(layer) && angular.isNumber(id)) {
                if (data[layer] && data[layer].modalInstance !== null && data[layer].id === id) {
                    data[layer].modalInstance.opened
                        .then(function () {
                            data[layer].modalInstance.close();
                        });
                }
            }
        },

        dismiss: function (layer) {
            var dismissPopup = function (layer) {
                if (data[layer] && data[layer].modalInstance !== null) {
                    data[layer].modalInstance.opened
                        .then(function () {
                            data[layer].modalInstance.dismiss();
                        });
                }
            };

            if (angular.isNumber(layer)) {
                dismissPopup(layer);
            } else {
                angular.forEach(data, function (layer, layerId) {
                    dismissPopup(layerId);
                });
            }
        },

        isDisplayed: function () {
            return data.some(function (layer) {
                return (layer.modalInstance !== null);
            });
        }
    };

    return Popup;
}]);

angular.module('WebUI').factory('Model', ['$q', '$http', '$cacheFactory', 'AuthenticationService', 'Tools', function ($q, $http, $cacheFactory, AuthenticationService, Tools) {
    var Model = {
        parameters: {},
        text: {},
        menu: {},

        getParameters: function (config) {
            var promise, url;

            config = config || {};

            url = angular.isString(config.parameterName) ? 'Parameters/' + config.parameterName : 'Parameters';

            if (config.removeCache) {
                $cacheFactory.get('$http').remove(url);
            }

            promise = $http.get(url);

            promise
                .then(function (response) {
                    var addRestoreFromCache = function (parameters, path) {
                        path = path || '';
                        angular.forEach(parameters, function (parameter, key) {
                            if (angular.isArray(parameter)) {
                                angular.forEach(parameter, function (arrayParameter, arrayKey) {
                                    addRestoreFromCache(arrayParameter, path + key + '[' + arrayKey + '].');
                                });
                            } else if (!parameter.restoreFromCache){
                                parameter.restoreFromCache = function () {
                                    Model.restoreParametersFromCache(path + key);
                                };
                            }
                        });
                    };

                    angular.merge(Model.parameters, response.data);

                    if (config.removeCache) {
                        Model.updateParametersCache();
                    }

                    addRestoreFromCache(Model.parameters);
                });

            return promise;
        },

        getText: function () {
            var promise = $http.get('Text');

            promise
                .then(function (response) {
                    angular.merge(Model.text, response.data);
                    Tools.updateLocales(Model.text);
                });

            return promise;
        },

        getMenu: function () {
            var promise = $http.get('Menu');

            promise
                .then(function (response) {
                    angular.merge(Model.menu, response.data);
                });

            return promise;
        },

        getAll: function () {
            var allowedFiles = [];

            if (AuthenticationService.isLogged()) {
                allowedFiles.push(Model.getText(), Model.getParameters(), Model.getMenu());
            } else {
                allowedFiles.push(Model.getText(), Model.getParameters());
            }

            return $q.all(allowedFiles);
        },

        updateParametersCache: function (parametersSource) {
            var parametersCache = $cacheFactory.get('$http').get('Parameters');
            var parametersFromCache = angular.isArray(parametersCache) ? angular.fromJson(parametersCache[1]) : angular.fromJson(parametersCache);
            var parametersFromSource = parametersSource || Model.parameters;

            Tools.mergeObjectsByKeyName(parametersFromCache, parametersFromSource, 'value');

            if (angular.isArray(parametersCache)) {
                parametersCache[1] = angular.toJson(parametersFromCache);
            } else {
                parametersCache = angular.toJson(parametersFromCache);
            }

            $cacheFactory.get('$http').put('Parameters', parametersCache);
        },

        getParametersFromCache: function (parameter) {
            var parametersCache = $cacheFactory.get('$http').get('Parameters');
            parametersCache = angular.isArray(parametersCache) ? angular.fromJson(parametersCache[1]) : angular.fromJson(parametersCache);

            if (angular.isString(parameter)) {
                parametersCache = parametersCache[parameter] ? parametersCache[parameter].value : null;
            }

            return parametersCache;
        },

        restoreParametersFromCache: function (parameter) {
            var parameterFromModel = (angular.isString(parameter)) ? Tools.getPropertyByString(Model.parameters, parameter) : Model.parameters;
            var parameterFromCache = (angular.isString(parameter)) ? Tools.getPropertyByString(Model.getParametersFromCache(), parameter) : Model.getParametersFromCache();

            Tools.mergeObjectsByKeyName(parameterFromModel, parameterFromCache, 'value');
        },

        removeParameters: function (removeCache) {
            Model.parameters = {};

            if (removeCache) {
                $cacheFactory.get('$http').remove('Parameters');
            }
        },

        removeText: function (removeCache) {
            Model.text = {};

            if (removeCache) {
                $cacheFactory.get('$http').remove('Text');
            }
        },

        removeMenu: function (removeCache) {
            Model.menu = {};

            if (removeCache) {
                $cacheFactory.get('$http').remove('Menu');
            }
        },

        removeAll: function (removeCache) {
            Model.removeParameters(removeCache);
            Model.removeText(removeCache);
            Model.removeMenu(removeCache);
        }
    };

    return Model;
}]);

angular.module('WebUI').factory('HTTPInterceptor', ['$q', '$window', '$rootScope', 'Config', function ($q, $window, $rootScope, Config) {
    var HTTPInterceptor = {
        request: function (config) {
            config.headers = config.headers || {};

            if ($window.sessionStorage.token) {
                config.headers.Authorization = 'Bearer ' + $window.sessionStorage.token;
            }

            if (!config.hasOwnProperty('timeout')) {
                config.timeout = (config.method === 'GET') ? Config.timers.defaultHttpGetTimeout : Config.timers.defaultHttpTimeout;
                config.timeout *= 1000;
            }

            return config || $q.when(config);
        },

        requestError: function (config) {
            return $q.reject(config);
        },

        response: function (response) {
            if (response && response.config && response.config.method === 'POST' && !response.config.hasOwnProperty('restartLogoutTimer')) {
                response.config.restartLogoutTimer = true;
            }

            $rootScope.$broadcast('Connection::Success', angular.copy(response));
            return response || $q.when(response);
        },

        responseError: function (response) {
            if (response && response.status >= 0) {
                if (response.config && (response.config.method === 'GET') || (response.config.method === 'POST')) {
                    $rootScope.$broadcast('Connection::Error', angular.copy(response));
                }
            }

            return $q.reject(response);
        }
    };

    return HTTPInterceptor;
}]);

angular.module('WebUI').factory('LogoutTimer', ['$http', '$location', '$q', '$window', '$timeout', '$log', 'State', 'Status', 'AuthenticationService', 'Model', 'Navigation', function ($http, $location, $q, $window, $timeout, $log, State, Status, AuthenticationService, Model, Navigation) {
    var data = {
        logoutTimeout: null,

        logoutInProgress: false,

        logoutFunction: function () {
            var deferred = $q.defer();

            if (AuthenticationService.isLogged()) {
                $log.info('Logging out.');

                State.stop();
                Status.keepAlive.stop();

                $http.post('Logout')
                    .then(function () {
                        $window.sessionStorage.clear();
                        Model.removeAll(false);
                        Navigation.History.remove();
                        State.clear();
                        $location.path('Login');

                        deferred.resolve();
                    })
                    .catch(function () {
                        deferred.reject();
                    });
            } else {
                deferred.reject();
            }

            return deferred.promise;
        }
    };

    var LogoutTimer = {
        restart: function () {
            if (AuthenticationService.isLogged() && !data.logoutInProgress && !LogoutTimer.restart.disabled) {
                LogoutTimer.cancel();
                data.logoutTimeout = $timeout(function () {
                    LogoutTimer.logout();
                }, Model.parameters.LogoutTimer.value * 60 * 1000);

                $log.info('Logout timer started: %d min.', Model.parameters.LogoutTimer.value);
            }
        },

        cancel: function () {
            if (data.logoutTimeout) {
                $log.info('Logout timer stopped.');

                $timeout.cancel(data.logoutTimeout);
                data.logoutTimeout = null;
            }
        },

        logout: function () {
            var deferred = $q.defer();

            if (!data.logoutInProgress) {
                data.logoutInProgress = true;
                LogoutTimer.cancel();
                data.logoutFunction()
                    .then(deferred.resolve)
                    .catch(deferred.reject)
                    .finally(function () {
                        data.logoutInProgress = false;
                    });
            } else {
                deferred.reject();
            }

            return deferred.promise;
        }
    };

    return LogoutTimer;
}]);

angular.module('WebUI').factory('Navigation', ['$location', '$window', 'Model', 'AuthenticationService', 'Config', function ($location, $window, Model, AuthenticationService, Config) {
    var isMenuEnabled = function (menuItem) {
        return function () {
            return Model.getParametersFromCache(menuItem) === '1';
        };
    };

    var data = {
        history: [],

        conditions: {
            isMenuSettingsEnabled: isMenuEnabled('MenuSettings'),
            isMenuNetworkEnabled: isMenuEnabled('MenuNetwork'),
            isMenuIPEnabled: isMenuEnabled('MenuIP'),
            isMenuLANEnabled: isMenuEnabled('MenuLAN'),
            isMenuTelephonyEnabled: isMenuEnabled('MenuTelephony'),
            isMenuVoIPEnabled: isMenuEnabled('MenuVoIP'),
            isMenuConnectionsEnabled: isMenuEnabled('MenuConnections'),
            isMenuAudioEnabled: isMenuEnabled('MenuAudio'),
            isMenuPhoneSystemEnabled: isMenuEnabled('MenuPhoneSystem'),
            isMenuCallSettingsEnabled: isMenuEnabled('MenuCallSettings'),
            isMenuCallDivertEnabled: isMenuEnabled('MenuCallDivert'),
            isMenuDoNotDisturbEnabled: isMenuEnabled('MenuDoNotDisturb'),
            isMenuDiallingPlansEnabled: isMenuEnabled('MenuDiallingPlans'),
            isMenuVoicemailEnabled: isMenuEnabled('MenuVoicemail'),
            isMenuOnlineDirectoriesEnabled: isMenuEnabled('MenuOnlineDirectories'),
            isMenuPublicEnabled: isMenuEnabled('MenuPublic'),
            isMenuCorporateEnabled: isMenuEnabled('MenuCorporate'),
            isMenuXMLEnabled: isMenuEnabled('MenuXML'),
            isMenuOnlineServicesEnabled: isMenuEnabled('MenuOnlineServices'),
            isMenuWebcamEnabled: isMenuEnabled('MenuWebcam'),
            isMenuDoorStationEnabled: isMenuEnabled('MenuDoorStation'),
            isMenuDeskPhoneEnabled: isMenuEnabled('MenuDeskPhone'),
            isMenuMainMenuEnabled: isMenuEnabled('MenuMainMenu'),
            isMenuRingtonesEnabled: isMenuEnabled('MenuRingtones'),
            isMenuDisplayEnabled: isMenuEnabled('MenuDisplay'),
            isMenuPicturesEnabled: isMenuEnabled('MenuPictures'),
            isMenuMessageNotificationEnabled: isMenuEnabled('MenuMessageNotification'),
            isMenuKeysAndLEDsEnabled: isMenuEnabled('MenuKeysAndLEDs'),
            isMenuEventsEnabled: isMenuEnabled('MenuEvents'),
            isMenuWebConfiguratorEnabled: isMenuEnabled('MenuWebConfigurator'),
            isMenuSystemEnabled: isMenuEnabled('MenuSystem'),
            isMenuPhoneWebServerEnabled: isMenuEnabled('MenuPhoneWebServer'),
            isMenuProvisioningAndConfigurationEnabled: isMenuEnabled('MenuProvisioningConfiguration'),
            isMenuSecurityEnabled: isMenuEnabled('MenuSecurity'),
            isMenuDateAndTimeEnabled: isMenuEnabled('MenuDateAndTime'),
            isMenuFirmwareUpdateEnabled: isMenuEnabled('MenuFirmwareUpdate'),
            isMenuSaveAndRestoreEnabled: isMenuEnabled('MenuSaveAndRestore'),
            isMenuRebootAndResetEnabled: isMenuEnabled('MenuRebootAndReset'),
            isMenuStatusEnabled: isMenuEnabled('MenuStatus'),
            isMenuDeviceEnabled: isMenuEnabled('MenuDevice'),
            isMenuStatusConnectionsEnabled: isMenuEnabled('MenuStatusConnections'),
            isMenuSIPProtocolEnabled: isMenuEnabled('MenuSIPProtocol'),
            isMenuPCAPLoggingEnabled: isMenuEnabled('MenuPCAPLogging'),
            isMenuStorageAllocationEnabled: isMenuEnabled('MenuStorageAllocation'),
            isMenuExpertEnabled: isMenuEnabled('MenuExpert'),
            isMenuExpertNetworkEnabled: isMenuEnabled('MenuExpertNetwork'),
            isMenuSwitchSettingsEnabled: isMenuEnabled('MenuSwitchSettings'),
            isMenuCLIEnabled: isMenuEnabled('MenuCLI'),
            isMenuExpertAudioEnabled: isMenuEnabled('MenuExpertAudio'),
            isMenuAdjustmentEnabled: isMenuEnabled('MenuAdjustment'),
            isMenuAdaptiveJitterBufferEnabled: isMenuEnabled('MenuAdaptiveJitterBuffer'),
            isMenuDebuggingEnabled: isMenuEnabled('MenuDebugging'),
            isMenuRebootEnabled: isMenuEnabled('MenuReboot'),
            isMenuSystemLoggingEnabled: isMenuEnabled('MenuSystemLogging'),
            isMaxwell3: function () {
                return Model.getParametersFromCache('PhoneModel') === Config.phoneModel['Maxwell 3'];
            },
            isAdminLogged: function () {
                return AuthenticationService.isAdminLogged();
            },
            isAdminLoggedOrMaxwell3: function () {
                return data.conditions.isAdminLogged() || data.conditions.isMaxwell3();
            },
            isPhoneSystemStandard: function () {
                return Model.getParametersFromCache('PhoneSystem') === Config.phoneSystem['STANDARD'];
            }
        }
    };

    var Navigation = {
        History: {
            update: function () {
                var page = $location.path().substr(1);

                data.history.unshift(page);
                $window.sessionStorage.lastUrl = page;
            },

            get: function (index) {
                if (angular.isDefined(index)) {
                    if (data.history.length > index) {
                        return data.history[index];
                    } else {
                        return null;
                    }
                } else {
                    return angular.copy(data.history);
                }
            },

            remove: function () {
                data.history = [];
            },

            goToPreviousPage: function () {
                $location.path(data.history[1]);
            }
        },

        Menu: {
            checkMenuCondition: function (menuItem, ruleName) {
                var rules = angular.isArray(ruleName) ? ruleName : [ruleName];

                var checkCondition = function (condition) {
                    var result = false;

                    if (angular.isArray(condition)) {
                        result = condition.every(function (conditionName) {
                            return angular.isFunction(data.conditions[conditionName]) ? data.conditions[conditionName]() : data.conditions[conditionName];
                        });
                    } else {
                        result = angular.isFunction(data.conditions[condition]) ? data.conditions[condition]() : data.conditions[condition];
                    }

                    return result;
                };

                var menuConditionValid = false,
                    subMenuConditionsValid = false;

                menuConditionValid = rules.every(function (ruleName) {
                    return !menuItem[ruleName] || checkCondition(menuItem[ruleName]);
                });

                if (menuConditionValid) {
                    subMenuConditionsValid = !angular.isArray(menuItem.menu) || menuItem.menu.some(function (subMenu) {
                        return rules.every(function (ruleName) {
                            return !subMenu[ruleName] || checkCondition(subMenu[ruleName]);
                        });
                    });
                }

                return menuConditionValid && subMenuConditionsValid;
            },

            getUrl: function (menu, nextUrl) {
                var getFirstUrlFromMenu = function (menu) {
                    var firstUrl = null;

                    menu.some(function (menuItem) {
                        if (Navigation.Menu.checkMenuCondition(menuItem, ['showIf', 'enableIf'])) {
                            if (angular.isArray(menuItem.menu)) {
                                firstUrl = getFirstUrlFromMenu(menuItem.menu);
                            } else {
                                firstUrl = menuItem.url.substr(2);
                            }
                        }

                        return (firstUrl !== null);
                    });

                    return firstUrl;
                };

                var urlFromMenu = null;

                if (angular.isArray(menu) && menu.length > 0) {
                    if (nextUrl) {
                        menu.some(function (menuItem) {
                            if (menuItem.url && nextUrl.indexOf(menuItem.url.substr(2)) !== -1 && Navigation.Menu.checkMenuCondition(menuItem, ['showIf', 'enableIf'])) {
                                if (menuItem.url.substr(2) === nextUrl || menuItem.url.substr(2) + '/' === nextUrl) {
                                    if (angular.isArray(menuItem.menu)) {
                                        urlFromMenu = getFirstUrlFromMenu(menuItem.menu);
                                    } else {
                                        urlFromMenu = menuItem.url.substr(2);
                                    }
                                } else if (angular.isArray(menuItem.menu)) {
                                    urlFromMenu = Navigation.Menu.getUrl(menuItem.menu, nextUrl);
                                }
                            }

                            return (urlFromMenu !== null);
                        });
                    } else {
                        urlFromMenu = getFirstUrlFromMenu(menu);
                    }
                }

                return urlFromMenu;
            },

            checkActiveMenu: function (menu, activeMenu, currentPath) {
                return menu.some(function (element, index) {
                    var menuItemFound = false;

                    if ( (element.url && element.url.substr(2) === currentPath) ||
                         (element.menu && Navigation.Menu.checkActiveMenu(element.menu, activeMenu, currentPath)) ) {
                        activeMenu.push(index);
                        menuItemFound = true;
                    }

                    return menuItemFound;
                });
            }
        },

        getPageName: function (page) {
            if (angular.isDefined(page)) {
                return page.slice(page.lastIndexOf('/') + 1);
            } else {
                return $location.path().slice($location.path().lastIndexOf('/') + 1);
            }
        }
    };

    return Navigation;
}]);

angular.module('WebUI').factory('State', ['$http', '$q', '$interval', function ($http, $q, $interval) {
    var data = {
        intervals: [],

        get: function (stateName) {
            return $http.get('State/' + stateName, {cache: false})
                .then(function (response) {
                    State.state[stateName] = response.data[stateName];
                    return response;
                });
        },

        set: function (state) {
            return $http.post('State', state)
                .then(function (response) {
                    angular.forEach(state, function (stateValue, stateName) {
                        State.state[stateName] = stateValue;
                    });

                    return response;
                });
        },

        getStates: function (states) {
            var deferred = $q.defer();

            angular.forEach(states, function (state) {
                data.get(state.name)
                    .then(function (response) {
                        if (angular.isNumber(state.interval)) {
                            deferred.notify(response);
                        } else {
                            deferred.resolve(response);
                        }
                    });

                if (angular.isNumber(state.interval)) {
                    var interval = $interval(function () {
                        data.get(state.name)
                            .then(deferred.notify);
                    }, state.interval * 1000);

                    data.intervals.push({id: interval, name: state.name, deferred: deferred});
                }
            });

            return deferred.promise;
        },

        setStates: function (states) {
            var promises = [];

            angular.forEach(states, function (state) {
                promises.push(data.set(state));
            });

            return $q.all(promises);
        }
    };

    var State = {
        state: {},

        get: function (states) {
            states = angular.isArray(states) ? states : [states];

            var statesFiltered = states.filter(function (state) {
                return data.intervals.every(function (interval) {
                    return interval.name !== state.name;
                });
            });

            return data.getStates(statesFiltered);
        },

        set: function (states) {
            states = angular.isArray(states) ? states : [states];
            return data.setStates(states);
        },

        stop: function (stateName) {
            var stopState = function (stateName) {
                data.intervals = data.intervals.filter(function (interval) {
                    if ((angular.isUndefined(stateName) || interval.name === stateName) && interval.id) {
                        $interval.cancel(interval.id);
                        interval.deferred.reject();
                        return false;
                    } else {
                        return true;
                    }
                });
            };

            if (angular.isArray(stateName)) {
                angular.forEach(stateName, function (stateName) {
                    stopState(stateName);
                });
            } else {
                stopState(stateName);
            }
        },

        clear: function () {
            angular.copy({}, State.state);
        }
    };

    return State;
}]);

angular.module('WebUI').factory('Status', ['$http', '$interval', '$timeout', '$rootScope', '$window', 'Config', function ($http, $interval, $timeout, $rootScope, $window, Config) {
    var data = {
        interval: {
            checkOffline: null,
            keepAlive: null
        },
        reloadAfterOffline: {
            timer: null,
            doReload: false
        },
        popupData: {}
    };

    var checkOffline = function (config) {
        return $http.get('Status', {cache: false, timeout: Config.timers.checkOfflineTimeout * 1000})
            .then(function () {
                if (!Status.status || Status.status === 'offline') {
                    Status.status = 'online';
                    $rootScope.$broadcast('Connection::Online', data.popupData);
                }

                if (data.reloadAfterOffline.timer) {
                    $timeout.cancel(data.reloadAfterOffline.timer);
                    data.reloadAfterOffline.timer = null;
                }

                if (data.reloadAfterOffline.doReload) {
                    $window.location.reload();
                }
            })
            .catch(function (response) {
                if (response && response.status === -1) {
                    if ((!Status.status || Status.status === 'online') && (config.startOnce || data.interval.checkOffline)) {
                        Status.status = 'offline';
                        $rootScope.$broadcast('Connection::Offline', data.popupData);

                        if (!data.reloadAfterOffline.timer) {
                            data.reloadAfterOffline.timer = $timeout(function () {
                                data.reloadAfterOffline.doReload = true;
                                data.reloadAfterOffline.timer = null;
                            }, Config.timers.reloadAfterOfflineTimeout * 1000);
                        }
                    }
                }
            });
    };

    var keepAlive = function (config) {
        return $http.post('Status', {alive: true}, {restartLogoutTimer: false});
    };

    var Status = {
        status: null,

        checkOffline: {
            start: function () {
                Status.checkOffline.stop();
                data.interval.checkOffline = $interval(checkOffline, Config.timers.checkOfflineInterval * 1000, 0, true, {startOnce: false});
            },

            startOnce: function () {
                return checkOffline({startOnce: true});
            },

            stop: function () {
                if (data.interval.checkOffline !== null) {
                    $interval.cancel(data.interval.checkOffline);
                    data.interval.checkOffline = null;
                }
            }
        },

        keepAlive: {
            start: function () {
                Status.keepAlive.stop();
                data.interval.keepAlive = $interval(keepAlive, Config.timers.keepAliveInterval * 1000, 0, true, {startOnce: false});
            },

            startOnce: function () {
                return keepAlive({startOnce: true});
            },

            stop: function () {
                if (data.interval.keepAlive !== null) {
                    $interval.cancel(data.interval.keepAlive);
                    data.interval.keepAlive = null;
                }
            }
        }
    };

    return Status;
}]);

angular.module('WebUI').constant('Config', {
    switch: {
        yes: '1',
        no: '0'
    },

    checkbox: {
        checked: '1',
        unchecked: '0'
    },

    button: {
        ok: 'ok',
        cancel: 'cancel',
        yes: 'yes',
        no: 'no'
    },

    flags: {
        ui_lock: 1,
        required: 32
    },

    popup: {
        layer: {
            default: 0,
            important: 1
        },

        priority: {
            info: 1,
            user: 2,
            error: 3
        },

        error: {
            dataIsMissing: 1,
            otherPopupIsShown: 2
        }
    },

    formatDate: {
        'dd.mm.yyyy': 'dd.MM.yyyy',
        'mm.dd.yyyy': 'MM.dd.yyyy'
    },

    formatTime: {
		'12h': 'hh:mm a',
		'24h': 'HH:mm'
    },

    defaultUrl: 'Settings',

    statesToPoll: {
        Connections: [{
            name: 'SIP.Account.0.RegistrationState',
            interval: 5
        },
        {
            name: 'SIP.Account.1.RegistrationState',
            interval: 5
        },
        {
            name: 'SIP.Account.2.RegistrationState',
            interval: 5
        },
        {
            name: 'SIP.Account.3.RegistrationState',
            interval: 5
        }],
        StatusConnections: [{
            name: 'SIP.Account.0.RegistrationState',
            interval: 5
        },
        {
            name: 'SIP.Account.1.RegistrationState',
            interval: 5
        },
        {
            name: 'SIP.Account.2.RegistrationState',
            interval: 5
        },
        {
            name: 'SIP.Account.3.RegistrationState',
            interval: 5
        }],
	    Device: [{
	        name: 'NET.MAC.Address'
	    }],
        SIPProtocol: [{
	        name: 'Network.SIP_PCAP'
	    }],
        PCAPLogging: [{
	        name: 'Network.PCAP'
	    }]
    },

    timers: {
        checkOfflineInterval: 5,
        checkOfflineTimeout: 5,
        defaultHttpGetTimeout: 10,
        defaultHttpTimeout: 60,
        keepAliveInterval: 120,
        reloadAfterOfflineTimeout: 30
    },

    codecs: {
        'G.711 ulaw': 0,
        'G.723.1': 4,
        'G.711 alaw': 8,
        'G.722': 9,
        'iLBC': 109,
        'G.726-32': 115
    },

    phoneSystem: {
        'STANDARD': '0',
        'GIGASET T640/T440': '1',
        'GIGASET T300/T500': '2',
        'HYBIRD 120 GE': '3',
        'STARFACE': '4',
        'BROADSOFT': '5',
        'TELES': '6'
    },

    phoneModel: {
        'Maxwell 3': '1',
        'Maxwell Basic': '2'
    },

    maxInputFieldLength: 255,

    maxUploadFileSize: 60 * 1024 * 1024
});
